@@ -43,8 +43,7 @@ async fn main() {
         platform: "linux/x86_64",
     };
 
-    let mut image_build_stream = docker
-        .build_image(build_image_options, None, None);
+    let mut image_build_stream = docker.build_image(build_image_options, None, None);
 
     while let Some(msg) = image_build_stream.next().await {
         println!("Message: {:?}", msg);
