@@ -1,7 +1,10 @@
 //! Removes old docker containers, images, volumes and networks
 
-use bollard::{container::PruneContainersOptions, image::PruneImagesOptions, network::PruneNetworksOptions, volume::PruneVolumesOptions};
 use bollard::Docker;
+use bollard::{
+    container::PruneContainersOptions, image::PruneImagesOptions, network::PruneNetworksOptions,
+    volume::PruneVolumesOptions,
+};
 use chrono::{Duration, Utc};
 
 use std::collections::HashMap;
@@ -18,25 +21,33 @@ async fn main() -> Result<(), Box<dyn std::error::Error + 'static>> {
     let mut prune_filters = HashMap::new();
     prune_filters.insert("until", vec![timestamp]);
 
-    let prune = docker.prune_containers(Some(PruneContainersOptions {
-        filters: prune_filters.clone()
-    })).await?;
+    let prune = docker
+        .prune_containers(Some(PruneContainersOptions {
+            filters: prune_filters.clone(),
+        }))
+        .await?;
 
     println!("{:?}", prune);
 
-    let prune = docker.prune_images(Some(PruneImagesOptions {
-        filters: prune_filters.clone()
-    })).await?;
+    let prune = docker
+        .prune_images(Some(PruneImagesOptions {
+            filters: prune_filters.clone(),
+        }))
+        .await?;
 
     println!("{:?}", prune);
 
-    let prune = docker.prune_volumes(None::<PruneVolumesOptions<String>>).await?;
+    let prune = docker
+        .prune_volumes(None::<PruneVolumesOptions<String>>)
+        .await?;
 
     println!("{:?}", prune);
 
-    let prune = docker.prune_networks(Some(PruneNetworksOptions {
-        filters: prune_filters.clone()
-    })).await?;
+    let prune = docker
+        .prune_networks(Some(PruneNetworksOptions {
+            filters: prune_filters.clone(),
+        }))
+        .await?;
 
     println!("{:?}", prune);
 
