@@ -2,20 +2,24 @@
 
 use chrono::{DateTime, Utc};
 use futures_core::Stream;
-use http::header::CONTENT_TYPE;
+use http::header::{CONNECTION, CONTENT_TYPE, UPGRADE};
 use http::request::Builder;
 use hyper::{body::Bytes, Body, Method};
 use serde::Serialize;
+use tokio::io::AsyncWrite;
+use tokio_util::codec::FramedRead;
 
 use std::cmp::Eq;
 use std::collections::HashMap;
 use std::fmt;
 use std::hash::Hash;
+use std::pin::Pin;
 
 use super::Docker;
 use crate::errors::Error;
 
 use crate::models::*;
+use crate::read::NewlineLogOutputDecoder;
 
 /// Parameters used in the [List Container API](Docker::list_containers())
 ///
@@ -44,7 +48,7 @@ use crate::models::*;
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct ListContainersOptions<T>
 where
     T: Into<String> + Eq + Hash + Serialize,
@@ -86,7 +90,7 @@ where
 ///     name: "my-new-container",
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct CreateContainerOptions<T>
 where
     T: Into<String> + Serialize,
@@ -96,7 +100,7 @@ where
 }
 
 /// This container's networking configuration.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "PascalCase")]
 #[allow(missing_docs)]
 pub struct NetworkingConfig<T: Into<String> + Hash + Eq> {
@@ -104,7 +108,7 @@ pub struct NetworkingConfig<T: Into<String> + Hash + Eq> {
 }
 
 /// Container to create.
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct Config<T>
 where
     T: Into<String> + Eq + Hash,
@@ -289,7 +293,7 @@ impl From<ContainerConfig> for Config<String> {
 /// StopContainerOptions{
 ///     t: 30,
 /// };
-#[derive(Debug, Copy, Clone, Default, Serialize)]
+#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize)]
 pub struct StopContainerOptions {
     /// Number of seconds to wait before killing the container
     pub t: i64,
@@ -306,7 +310,7 @@ pub struct StopContainerOptions {
 ///     detach_keys: "ctrl-^"
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 #[serde(rename_all = "camelCase")]
 pub struct StartContainerOptions<T>
 where
@@ -331,7 +335,7 @@ where
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Copy, Clone, Default, Serialize)]
+#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize)]
 pub struct RemoveContainerOptions {
     /// Remove the volumes associated with the container.
     pub v: bool,
@@ -352,7 +356,7 @@ pub struct RemoveContainerOptions {
 ///     condition: "not-running",
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct WaitContainerOptions<T>
 where
     T: Into<String> + Serialize,
@@ -362,6 +366,81 @@ where
     pub condition: T,
 }
 
+/// Results type for the [Attach Container API](Docker::attach_container())
+pub struct AttachContainerResults {
+    /// [Log Output](LogOutput) enum, wrapped in a Stream.
+    pub output: Pin<Box<dyn Stream<Item = Result<LogOutput, Error>> + Send>>,
+    /// Byte writer to container
+    pub input: Pin<Box<dyn AsyncWrite + Send>>,
+}
+
+impl fmt::Debug for AttachContainerResults {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "AttachContainerResults")
+    }
+}
+
+/// Parameters used in the [Attach Container API](Docker::attach_container())
+///
+/// ## Examples
+///
+/// ```rust
+/// use bollard::container::AttachContainerOptions;
+///
+/// AttachContainerOptions::<String>{
+///     stdin: Some(true),
+///     stdout: Some(true),
+///     stderr: Some(true),
+///     stream: Some(true),
+///     logs: Some(true),
+///     detach_keys: Some("ctrl-c".to_string()),
+/// };
+/// ```
+#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize)]
+pub struct AttachContainerOptions<T>
+where
+    T: Into<String> + Serialize + Default,
+{
+    /// Attach to `stdin`
+    pub stdin: Option<bool>,
+    /// Attach to `stdout`
+    pub stdout: Option<bool>,
+    /// Attach to `stderr`
+    pub stderr: Option<bool>,
+    /// Stream attached streams from the time the request was made onwards.
+    pub stream: Option<bool>,
+    /// Replay previous logs from the container.
+    /// This is useful for attaching to a container that has started and you want to output everything since the container started.
+    /// If stream is also enabled, once all the previous output has been returned, it will seamlessly transition into streaming current output.
+    pub logs: Option<bool>,
+    /// Override the key sequence for detaching a container.
+    /// Format is a single character [a-Z] or ctrl-<value> where <value> is one of: a-z, @, ^, [, , or _.
+    #[serde(rename = "detachKeys")]
+    pub detach_keys: Option<T>,
+}
+
+/// Parameters used in the [Resize Container Tty API](Docker::resize_container_tty())
+///
+/// ## Examples
+///
+/// ```rust
+/// use bollard::container::ResizeContainerTtyOptions;
+///
+/// ResizeContainerTtyOptions {
+///     width: 50,
+///     height: 10,
+/// };
+/// ```
+#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize)]
+pub struct ResizeContainerTtyOptions {
+    /// Width of the TTY session in characters
+    #[serde(rename = "w")]
+    pub width: u16,
+    /// Height of the TTY session in characters
+    #[serde(rename = "h")]
+    pub height: u16,
+}
+
 /// Parameters used in the [Restart Container API](Docker::restart_container())
 ///
 /// ## Example
@@ -373,7 +452,7 @@ where
 ///     t: 30,
 /// };
 /// ```
-#[derive(Debug, Copy, Clone, Default, Serialize)]
+#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize)]
 pub struct RestartContainerOptions {
     /// Number of seconds to wait before killing the container.
     pub t: isize,
@@ -390,7 +469,7 @@ pub struct RestartContainerOptions {
 ///     size: false,
 /// };
 /// ```
-#[derive(Debug, Copy, Clone, Default, Serialize)]
+#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize)]
 pub struct InspectContainerOptions {
     /// Return the size of container as fields `SizeRw` and `SizeRootFs`
     pub size: bool,
@@ -407,7 +486,7 @@ pub struct InspectContainerOptions {
 ///     ps_args: "aux",
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 #[serde(rename_all = "camelCase")]
 pub struct TopOptions<T>
 where
@@ -435,7 +514,7 @@ fn is_zero(val: &i64) -> bool {
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct LogsOptions<T>
 where
     T: Into<String> + Serialize,
@@ -505,7 +584,7 @@ impl LogOutput {
 ///     one_shot: false,
 /// };
 /// ```
-#[derive(Debug, Copy, Clone, Default, Serialize)]
+#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize)]
 pub struct StatsOptions {
     /// Stream the output. If false, the stats will be output once and then it will disconnect.
     pub stream: bool,
@@ -515,7 +594,7 @@ pub struct StatsOptions {
 }
 
 /// Granular memory statistics for the container.
-#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 #[serde(untagged)]
 pub enum MemoryStatsStats {
@@ -526,7 +605,7 @@ pub enum MemoryStatsStats {
 /// Granular memory statistics for the container, v1 cgroups.
 ///
 /// Exposed in the docker library [here](https://github.com/moby/moby/blob/40d9e2aff130b42ba0f83d5238b9b53184c8ab3b/daemon/daemon_unix.go#L1436).
-#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 #[serde(deny_unknown_fields)]
 pub struct MemoryStatsStatsV1 {
@@ -562,14 +641,14 @@ pub struct MemoryStatsStatsV1 {
     pub total_pgmajfault: u64,
     pub total_pgpgin: u64,
     pub hierarchical_memsw_limit: Option<u64>, // only on OSX
-    pub shmem: Option<u64>, // only on linux kernel > 4.15.0-1106
-    pub total_shmem: Option<u64>, // only on linux kernel > 4.15.0-1106
+    pub shmem: Option<u64>,                    // only on linux kernel > 4.15.0-1106
+    pub total_shmem: Option<u64>,              // only on linux kernel > 4.15.0-1106
 }
 
 /// Granular memory statistics for the container, v2 cgroups.
 ///
 /// Exposed in the docker library [here](https://github.com/moby/moby/blob/40d9e2aff130b42ba0f83d5238b9b53184c8ab3b/daemon/daemon_unix.go#L1542).
-#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 #[serde(deny_unknown_fields)]
 pub struct MemoryStatsStatsV2 {
@@ -607,7 +686,7 @@ pub struct MemoryStatsStatsV2 {
 }
 
 /// General memory statistics for the container.
-#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct MemoryStats {
     pub stats: Option<MemoryStatsStats>,
@@ -623,7 +702,7 @@ pub struct MemoryStats {
 }
 
 /// Process ID statistics for the container.
-#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct PidsStats {
     pub current: Option<u64>,
@@ -631,7 +710,7 @@ pub struct PidsStats {
 }
 
 /// I/O statistics for the container.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct BlkioStats {
     pub io_service_bytes_recursive: Option<Vec<BlkioStatsEntry>>,
@@ -645,7 +724,7 @@ pub struct BlkioStats {
 }
 
 /// File I/O statistics for the container.
-#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct StorageStats {
     pub read_count_normalized: Option<u64>,
@@ -655,7 +734,7 @@ pub struct StorageStats {
 }
 
 /// Statistics for the container.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct Stats {
     pub read: DateTime<Utc>,
@@ -674,7 +753,7 @@ pub struct Stats {
 }
 
 /// Network statistics for the container.
-#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct NetworkStats {
     pub rx_dropped: u64,
@@ -688,7 +767,7 @@ pub struct NetworkStats {
 }
 
 /// CPU usage statistics for the container.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct CPUUsage {
     pub percpu_usage: Option<Vec<u64>>,
@@ -698,7 +777,7 @@ pub struct CPUUsage {
 }
 
 /// CPU throttling statistics.
-#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct ThrottlingData {
     pub periods: u64,
@@ -707,7 +786,7 @@ pub struct ThrottlingData {
 }
 
 /// General CPU statistics for the container.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct CPUStats {
     pub cpu_usage: CPUUsage,
@@ -716,7 +795,7 @@ pub struct CPUStats {
     pub throttling_data: ThrottlingData,
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[allow(missing_docs)]
 pub struct BlkioStatsEntry {
     pub major: u64,
@@ -736,7 +815,7 @@ pub struct BlkioStatsEntry {
 ///     signal: "SIGINT",
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct KillContainerOptions<T>
 where
     T: Into<String> + Serialize,
@@ -759,7 +838,7 @@ where
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 #[serde(rename_all = "PascalCase")]
 pub struct UpdateContainerOptions<T>
 where
@@ -943,7 +1022,7 @@ where
 ///     name: "my_new_container_name"
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct RenameContainerOptions<T>
 where
     T: Into<String> + Serialize,
@@ -968,7 +1047,7 @@ where
 ///     filters
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct PruneContainersOptions<T>
 where
     T: Into<String> + Eq + Hash + Serialize,
@@ -997,7 +1076,7 @@ where
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 #[serde(rename_all = "camelCase")]
 pub struct UploadToContainerOptions<T>
 where
@@ -1022,7 +1101,7 @@ where
 ///     path: "/opt",
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct DownloadFromContainerOptions<T>
 where
     T: Into<String> + Serialize,
@@ -1044,7 +1123,7 @@ impl Docker {
     ///
     /// # Returns
     ///
-    ///  - Vector of [ContainerSummaryInner](ContainerSummaryInner), wrapped in a Future.
+    ///  - Vector of [ContainerSummary](ContainerSummary), wrapped in a Future.
     ///
     /// # Examples
     ///
@@ -1070,7 +1149,7 @@ impl Docker {
     pub async fn list_containers<'de, T>(
         &self,
         options: Option<ListContainersOptions<T>>,
-    ) -> Result<Vec<ContainerSummaryInner>, Error>
+    ) -> Result<Vec<ContainerSummary>, Error>
     where
         T: Into<String> + Eq + Hash + Serialize,
     {
@@ -1332,6 +1411,113 @@ impl Docker {
         self.process_into_stream(req)
     }
 
+    /// ---
+    ///
+    /// # Attach Container
+    ///
+    /// Attach to a container to read its output or send it input. You can attach to the
+    /// same container multiple times and you can reattach to containers that have been detached.
+    ///
+    /// # Arguments
+    ///
+    /// - Container name as string slice.
+    /// - Optional [Attach Container Options](AttachContainerOptions) struct.
+    ///
+    /// # Returns
+    ///
+    ///  - [AttachContainerResults](AttachContainerResults) wrapped in a Future.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # use bollard::Docker;
+    /// # let docker = Docker::connect_with_http_defaults().unwrap();
+    ///
+    /// use bollard::container::AttachContainerOptions;
+    ///
+    /// let options = Some(AttachContainerOptions::<String>{
+    ///     stdin: Some(true),
+    ///     stdout: Some(true),
+    ///     stderr: Some(true),
+    ///     stream: Some(true),
+    ///     logs: Some(true),
+    ///     detach_keys: Some("ctrl-c".to_string()),
+    /// });
+    ///
+    /// docker.attach_container("hello-world", options);
+    /// ```
+    pub async fn attach_container<T>(
+        &self,
+        container_name: &str,
+        options: Option<AttachContainerOptions<T>>,
+    ) -> Result<AttachContainerResults, Error>
+    where
+        T: Into<String> + Serialize + Default,
+    {
+        let url = format!("/containers/{}/attach", container_name);
+
+        let req = self.build_request(
+            &url,
+            Builder::new()
+                .method(Method::POST)
+                .header(CONNECTION, "Upgrade")
+                .header(UPGRADE, "tcp"),
+            options,
+            Ok(Body::empty()),
+        );
+
+        let (read, write) = self.process_upgraded(req).await?;
+        let log = FramedRead::new(read, NewlineLogOutputDecoder::new());
+
+        Ok(AttachContainerResults {
+            output: Box::pin(log),
+            input: Box::pin(write),
+        })
+    }
+
+    /// ---
+    ///
+    /// # Resize container tty
+    ///
+    /// Resize the container's TTY.
+    ///
+    /// # Arguments
+    ///
+    /// - Container name as string slice.
+    /// - [Resize Container Tty Options](ResizeContainerTtyOptions) struct.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # use bollard::Docker;
+    /// # let docker = Docker::connect_with_http_defaults().unwrap();
+    ///
+    /// use bollard::container::ResizeContainerTtyOptions;
+    ///
+    /// let options = ResizeContainerTtyOptions {
+    ///     width: 50,
+    ///     height: 20,
+    /// };
+    ///
+    /// docker.resize_container_tty("hello-world", options);
+    /// ```
+    pub async fn resize_container_tty(
+        &self,
+        container_name: &str,
+        options: ResizeContainerTtyOptions,
+    ) -> Result<(), Error> {
+        let url = format!("/containers/{}/resize", container_name);
+
+        let req = self.build_request(
+            &url,
+            Builder::new().method(Method::POST),
+            Some(options),
+            Ok(Body::empty()),
+        );
+
+        self.process_into_unit(req).await
+    }
+
     /// ---
     ///
     /// # Restart Container
