@@ -39,7 +39,7 @@ use std::hash::Hash;
 ///   ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 #[serde(rename_all = "camelCase")]
 pub struct CreateImageOptions<T>
 where
@@ -91,7 +91,7 @@ where
 /// };
 /// ```
 ///
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct ListImagesOptions<T>
 where
     T: Into<String> + Eq + Hash + Serialize,
@@ -135,7 +135,7 @@ where
 /// };
 /// ```
 ///
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct PruneImagesOptions<T>
 where
     T: Into<String> + Eq + Hash + Serialize,
@@ -179,7 +179,7 @@ where
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct SearchImagesOptions<T>
 where
     T: Into<String> + Eq + Hash + Serialize,
@@ -209,7 +209,7 @@ where
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Copy, Clone, Default, Serialize)]
+#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize)]
 pub struct RemoveImageOptions {
     /// Remove the image even if it is being used by stopped containers or has other tags.
     pub force: bool,
@@ -238,7 +238,7 @@ pub struct RemoveImageOptions {
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct TagImageOptions<T>
 where
     T: Into<String> + Serialize,
@@ -268,7 +268,7 @@ where
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct PushImageOptions<T>
 where
     T: Into<String> + Serialize,
@@ -298,7 +298,7 @@ where
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct CommitContainerOptions<T>
 where
     T: Into<String> + Serialize,
@@ -340,7 +340,7 @@ where
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize)]
 pub struct BuildImageOptions<T>
 where
     T: Into<String> + Eq + Hash + Serialize,
@@ -418,7 +418,7 @@ where
 ///     ..Default::default()
 /// };
 /// ```
-#[derive(Debug, Copy, Clone, Default, Serialize)]
+#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize)]
 pub struct ImportImageOptions {
     /// Suppress progress details during load.
     pub quiet: bool,
@@ -494,7 +494,7 @@ impl Docker {
     ///
     /// # Returns
     ///
-    ///  - [Build Info](BuildInfo), wrapped in an asynchronous
+    ///  - [Create Image Info](CreateImageInfo), wrapped in an asynchronous
     ///  Stream.
     ///
     /// # Examples
@@ -564,7 +564,7 @@ impl Docker {
     ///
     /// # Returns
     ///
-    ///  - [Image](Image), wrapped in a Future.
+    ///  - [ImageInspect](ImageInspect), wrapped in a Future.
     ///
     /// # Examples
     ///
@@ -576,7 +576,7 @@ impl Docker {
     ///
     /// docker.inspect_image("hello-world");
     /// ```
-    pub async fn inspect_image(&self, image_name: &str) -> Result<Image, Error> {
+    pub async fn inspect_image(&self, image_name: &str) -> Result<ImageInspect, Error> {
         let url = format!("/images/{}/json", image_name);
 
         let req = self.build_request(
@@ -1026,7 +1026,7 @@ impl Docker {
     {
         let url = "/build";
 
-        match serde_json::to_string(&credentials.unwrap_or_else(HashMap::new)) {
+        match serde_json::to_string(&credentials.unwrap_or_default()) {
             Ok(ser_cred) => {
                 let req = self.build_request(
                     url,
@@ -1142,7 +1142,7 @@ impl Docker {
         root_fs: Body,
         credentials: Option<HashMap<String, DockerCredentials>>,
     ) -> impl Stream<Item = Result<BuildInfo, Error>> {
-        match serde_json::to_string(&credentials.unwrap_or_else(HashMap::new)) {
+        match serde_json::to_string(&credentials.unwrap_or_default()) {
             Ok(ser_cred) => {
                 let req = self.build_request(
                     "/images/load",
