@@ -4,8 +4,6 @@ use futures_util::stream::TryStreamExt;
 use std::future::Future;
 use tokio::runtime::Runtime;
 
-use std;
-
 use bollard::auth::DockerCredentials;
 use bollard::container::*;
 use bollard::errors::Error;
@@ -92,9 +90,9 @@ where
     T: Future<Output = Result<(), Error>>,
 {
     rt.block_on(future)
-        .or_else(|e| {
+        .map_err(|e| {
             println!("{:?}", e);
-            Err(e)
+            e
         })
         .unwrap();
 }
@@ -165,6 +163,55 @@ pub async fn create_container_hello_world(
     Ok(())
 }
 
+#[allow(dead_code)]
+pub async fn create_shell_daemon(
+    docker: &Docker,
+    container_name: &'static str,
+) -> Result<(), Error> {
+    let image = if cfg!(windows) {
+        format!("{}nanoserver/iis", registry_http_addr())
+    } else {
+        format!("{}alpine", registry_http_addr())
+    };
+
+    let _ = &docker
+        .create_image(
+            Some(CreateImageOptions {
+                from_image: &image[..],
+                ..Default::default()
+            }),
+            None,
+            if cfg!(windows) {
+                None
+            } else {
+                Some(integration_test_registry_credentials())
+            },
+        )
+        .try_collect::<Vec<_>>()
+        .await?;
+
+    let result = &docker
+        .create_container(
+            Some(CreateContainerOptions {
+                name: container_name,
+            }),
+            Config {
+                image: Some(image),
+                open_stdin: Some(true),
+                ..Default::default()
+            },
+        )
+        .await?;
+
+    assert_ne!(result.id.len(), 0);
+
+    let _ = &docker
+        .start_container(container_name, None::<StartContainerOptions<String>>)
+        .await?;
+
+    Ok(())
+}
+
 #[allow(dead_code)]
 pub async fn create_daemon(docker: &Docker, container_name: &'static str) -> Result<(), Error> {
     let image = if cfg!(windows) {
