@@ -1,9 +1,10 @@
 #![type_length_limit = "2097152"]
 
 use bollard::container::{
-    Config, CreateContainerOptions, DownloadFromContainerOptions, InspectContainerOptions,
-    KillContainerOptions, ListContainersOptions, LogsOptions, PruneContainersOptions,
-    RemoveContainerOptions, RenameContainerOptions, RestartContainerOptions, StatsOptions,
+    AttachContainerOptions, AttachContainerResults, Config, CreateContainerOptions,
+    DownloadFromContainerOptions, InspectContainerOptions, KillContainerOptions,
+    ListContainersOptions, LogsOptions, PruneContainersOptions, RemoveContainerOptions,
+    RenameContainerOptions, ResizeContainerTtyOptions, RestartContainerOptions, StatsOptions,
     TopOptions, UpdateContainerOptions, UploadToContainerOptions, WaitContainerOptions,
 };
 use bollard::errors::Error;
@@ -12,6 +13,7 @@ use bollard::models::*;
 use bollard::Docker;
 
 use futures_util::stream::TryStreamExt;
+use tokio::io::AsyncWriteExt;
 use tokio::runtime::Runtime;
 
 use std::io::Write;
@@ -280,6 +282,104 @@ async fn kill_container_test(docker: Docker) -> Result<(), Error> {
     Ok(())
 }
 
+async fn attach_container_test(docker: Docker) -> Result<(), Error> {
+    create_shell_daemon(&docker, "integration_test_attach_container").await?;
+
+    let unique_string = "bollard_unique_string";
+    let AttachContainerResults { output, mut input } = docker
+        .attach_container(
+            "integration_test_attach_container",
+            Some(AttachContainerOptions::<String> {
+                stream: Some(true),
+                stdout: Some(true),
+                stdin: Some(true),
+                ..Default::default()
+            }),
+        )
+        .await?;
+
+    input
+        .write_all(format!("echo {}\n", unique_string).as_bytes())
+        .await?;
+    input.write_all("exit\n".as_bytes()).await?;
+
+    let log = match tokio::time::timeout(tokio::time::Duration::from_secs(2), output.try_collect())
+        .await
+    {
+        Ok(res) => res?,
+        Err(_) => {
+            docker
+                .kill_container(
+                    "integration_test_attach_container",
+                    None::<KillContainerOptions<String>>,
+                )
+                .await?;
+            vec![]
+        }
+    };
+
+    let _ = &docker
+        .wait_container(
+            "integration_test_attach_container",
+            None::<WaitContainerOptions<String>>,
+        )
+        .try_collect::<Vec<_>>()
+        .await?;
+
+    let _ = &docker
+        .remove_container(
+            "integration_test_attach_container",
+            None::<RemoveContainerOptions>,
+        )
+        .await?;
+
+    let input_found = log
+        .iter()
+        .any(|val| val.to_string().contains(unique_string));
+
+    assert!(input_found);
+
+    Ok(())
+}
+
+async fn resize_container_test(docker: Docker) -> Result<(), Error> {
+    create_shell_daemon(&docker, "integration_test_resize_container_tty").await?;
+
+    docker
+        .resize_container_tty(
+            "integration_test_resize_container_tty",
+            ResizeContainerTtyOptions {
+                width: 50,
+                height: 50,
+            },
+        )
+        .await?;
+
+    let _ = &docker
+        .kill_container(
+            "integration_test_resize_container_tty",
+            None::<KillContainerOptions<String>>,
+        )
+        .await?;
+
+    let _ = &docker
+        .wait_container(
+            "integration_test_resize_container_tty",
+            None::<WaitContainerOptions<String>>,
+        )
+        .try_collect::<Vec<_>>()
+        .await?;
+
+    let _ = &docker
+        .remove_container(
+            "integration_test_resize_container_tty",
+            None::<RemoveContainerOptions>,
+        )
+        .await?;
+
+    Ok(())
+}
+
 async fn update_container_test(docker: Docker) -> Result<(), Error> {
     let update_options = UpdateContainerOptions::<String> {
         memory: Some(314572800),
@@ -416,7 +516,7 @@ async fn prune_containers_test(docker: Docker) -> Result<(), Error> {
     assert_eq!(
         0,
         result
-            .into_iter()
+            .iter()
             .filter(|r| vec![
                 "bollard",
                 "registry:2",
@@ -425,8 +525,7 @@ async fn prune_containers_test(docker: Docker) -> Result<(), Error> {
             ]
             .into_iter()
             .all(|v| v != r.image.as_ref().unwrap()))
-            .collect::<Vec<_>>()
-            .len()
+            .count()
     );
 
     Ok(())
@@ -774,3 +873,13 @@ fn integration_test_inspect_containers() {
 fn integration_test_mount_volume_containers() {
     connect_to_docker_and_run!(mount_volume_container_test);
 }
+
+#[test]
+fn integration_test_attach_container() {
+    connect_to_docker_and_run!(attach_container_test);
+}
+
+#[test]
+fn integration_test_resize_container_tty() {
+    connect_to_docker_and_run!(resize_container_test);
+}
